from django.conf.urls import url, include
from django.urls import path

import response.slack.views as views

urlpatterns = [
    path('slash_command', views.slash_command, name='slash_command'),
    path('action', views.action, name='action'),
    path('event', views.event, name='event'),
    path('cron_minute', views.cron_minute, name='cron_minute'),
]



from django.contrib import admin
from response.slack.models import HeadlinePost, CommsChannel, Notification, UserStats, PinnedMessage


# Register your models here.
admin.site.register(HeadlinePost)
admin.site.register(CommsChannel)
admin.site.register(Notification)
admin.site.register(UserStats)
admin.site.register(PinnedMessage)



from django.apps import AppConfig
from django.conf import settings as site_settings


class ResponseConfig(AppConfig):
    name = 'response'

    def ready(self):
        from .slack import (settings,
                            signals,
                            action_handlers,
                            event_handlers,
                            incident_commands,
                            incident_notifications,
                            dialog_handlers)

        site_settings.RESPONSE_LOGIN_REQUIRED = getattr(
            site_settings,
            'RESPONSE_LOGIN_REQUIRED',
            True,
        )



# Generated by Django 2.2.3 on 2019-09-04 13:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('response', '0002_timelineevent'),
    ]

    operations = [
        migrations.AddField(
            model_name='pinnedmessage',
            name='timeline_event',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='response.TimelineEvent'),
        ),
        migrations.AlterField(
            model_name='timelineevent',
            name='event_type',
            field=models.CharField(help_text='Type of event.', max_length=10),
        ),
    ]



# Generated by Django 2.2.3 on 2019-09-04 17:04

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('response', '0003_auto_20190904_1349'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timelineevent',
            name='metadata',
            field=jsonfield.fields.JSONField(help_text='Additional fields that can be added by other event types', null=True),
        ),
    ]



import factory
from faker import Factory

from response.core.models import TimelineEvent

from .user import ExternalUserFactory

faker = Factory.create()

class TimelineEventFactory(factory.DjangoModelFactory):
    class Meta:
        model = TimelineEvent

    incident = factory.SubFactory('tests.factories.IncidentFactory')

    timestamp = factory.LazyFunction(
        lambda: faker.date_time_between(start_date="-3d", end_date="now", tzinfo=None)
    )
    text = factory.LazyFunction(
        lambda: faker.paragraph(nb_sentences=5, variable_nb_sentences=True)
    )
    event_type = "text"



# Generated by Django 2.2.3 on 2019-09-06 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('response', '0004_auto_20190904_1704'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timelineevent',
            name='event_type',
            field=models.CharField(help_text='Type of event.', max_length=30),
        ),
    ]



from django.shortcuts import render
from django.http import HttpRequest, HttpResponse, Http404

from response.core.models import Incident
from response.slack.models import PinnedMessage, UserStats
from response.decorators import response_login_required


@response_login_required
def incident_doc(request: HttpRequest, incident_id: str):

    try:
        incident = Incident.objects.get(pk=incident_id)
    except Incident.DoesNotExist:
        raise Http404("Incident does not exist")

    events = PinnedMessage.objects.filter(incident=incident).order_by('timestamp')
    user_stats = UserStats.objects.filter(incident=incident).order_by('-message_count')[:5]

    return render(request, template_name='incident_doc.html', context={
        "incident": incident,
        "events": events,
        "user_stats": user_stats,
    })



# Generated by Django 2.2.3 on 2019-08-19 15:07

import datetime
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('response', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimelineEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(default=datetime.datetime.now, help_text='Time of when this event occurred.')),
                ('text', models.TextField(help_text='Freeform text describing this event')),
                ('event_type', models.CharField(choices=[('text', 'Freeform text field')], help_text='Type of event.', max_length=10)),
                ('metadata', django.contrib.postgres.fields.jsonb.JSONField(help_text='Additional fields that can be added by other event types', null=True)),
                ('incident', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='response.Incident')),
            ],
        ),
    ]



import json
from datetime import datetime

from django.conf import settings
from response.core.models.incident import Incident

from response.slack.settings import INCIDENT_EDIT_DIALOG
from response.slack.dialog_builder import Dialog, Text, TextArea, SelectWithOptions, SelectFromUsers
from response.slack.models import HeadlinePost, CommsChannel

from response.slack.decorators import action_handler, ActionContext

import logging
logger = logging.getLogger(__name__)

@action_handler(HeadlinePost.CLOSE_INCIDENT_BUTTON)
def handle_close_incident(ac: ActionContext):
    ac.incident.end_time = datetime.now()
    ac.incident.save()


@action_handler(HeadlinePost.CREATE_COMMS_CHANNEL_BUTTON)
def handle_create_comms_channel(ac: ActionContext):
    if CommsChannel.objects.filter(incident=ac.incident).exists():
        return

    comms_channel = CommsChannel.objects.create_comms_channel(ac.incident)

    # Invite the bot to the channel
    try:
        settings.SLACK_CLIENT.invite_user_to_channel(settings.INCIDENT_BOT_ID, comms_channel.channel_id)
    except Exception as ex:
        logger.error(ex)

    # Un-invite the user who owns the Slack token,
    #   otherwise they'll be added to every incident channel
    slack_token_owner = settings.SLACK_CLIENT.get_slack_token_owner()
    if ac.incident.reporter != slack_token_owner:
        settings.SLACK_CLIENT.leave_channel(comms_channel.channel_id)

    # Update the headline post to link to this
    headline_post = HeadlinePost.objects.get(
        incident=ac.incident
    )
    headline_post.comms_channel = comms_channel
    headline_post.save()


@action_handler(HeadlinePost.EDIT_INCIDENT_BUTTON)
def handle_edit_incident_button(ac: ActionContext):
    dialog = Dialog(
        title=f"Edit Incident {ac.incident.pk}",
        submit_label="Save",
        state=ac.incident.pk,
        elements=[
            Text(label="Report", name="report", value=ac.incident.report),
            TextArea(label="Summary", name="summary", value=ac.incident.summary, optional=True, placeholder="Can you share any more details?"),
            TextArea(label="Impact", name="impact", value=ac.incident.impact, optional=True, placeholder="Who or what might be affected?", hint="Think about affected people, systems, and processes"),
            SelectFromUsers(label="Lead", name="lead", value=ac.incident.lead.external_id if ac.incident.lead else None, optional=True),
            SelectWithOptions([(s.capitalize(), i) for i, s in Incident.SEVERITIES], value=ac.incident.severity, label="Severity", name="severity", optional=True)
        ]
    )

    dialog.send_open_dialog(INCIDENT_EDIT_DIALOG, ac.trigger_id)
